import {Request, Response, Router} from "express";
import { v4 as uuid} from "uuid";
import {Recipe} from "../models/Recipe";
import * as passport from "passport";
import {validate} from "../utils/validation/validate";
import {validationRecipeSchema} from "../utils/validation/recipeValidationSchema";

export const recipesRouter = Router();

recipesRouter
    .get('', async (req, res) => {
        const recipes = await Recipe.findAll().catch(err => {
            res
                .status(500)
                .json({message: 'Wystąpił błąd, spróbuj ponownie pózniej.'})
        });

        // @ts-ignore
        if(recipes.length === 0) {
            res
                .status(400)
                .json( { message: "Aktualnie nie ma żadnych przepisów. "});
            return;
        }

        if(recipes) {
            res
                .status(200)
                .json(recipes);
        }
    })
    .get('/:name', async (req, res) => {
        const { name } = req.params;

        const oneRecipe = await Recipe.findOne({ where: { name } }).catch(err => {
            res
                .status(500)
                .json({message: 'Wystąpił błąd, spróbuj ponownie później.'})
        });

        if(oneRecipe) {
            res
                .status(200)
                .json( oneRecipe );
        } else {
            res
                .status(400)
                .json({ message: 'Przepis o takiej nazwie nie istnieje.'});
        }
    })
    .get('/search/:category', async (req, res) => {
        const { category } = req.params;

        const recipesWithCategory = await Recipe.findAll({ where: { category }}).catch(err => {
            res
                .status(500)
                .json({message: 'Wystąpił błąd, spróbuj ponownie później.'})
        });

        // @ts-ignore
        if(recipesWithCategory.length === 0) {
            res
                .status(400)
                .json({ message: 'Nie istnieją żadne przepisy w podanej kategorii.'})
            return;
        }

        if(recipesWithCategory) {
            res
                .status(200)
                .json(recipesWithCategory);
        }
    })
    // @ts-ignore
    .post('/add-recipe', [passport.authenticate("jwt", { session: false}), validate(validationRecipeSchema)], async (req, res) => {
        const {authorId, name, description, category, ingredients, todoList, imagePath} = req.body;



        const recipeNameExist = await Recipe.findOne({ where: { name } }).catch(err => {
            console.log(err);
        });

        if(recipeNameExist) {
            return res
                .status(400)
                .json( { message: "Przepis o podanej nazwie już istnieje."})

        }

        const id = uuid();
        // @ts-ignore
        const newRecipe = new Recipe({
            id,
            authorId,
            name,
            description,
            category,
            ingredients,
            todoList,
            imagePath
        });

        const saveRecipe = await newRecipe.save()
            .catch(err => {
                console.log(err);
                res.status(500)
                    .json({ message: "Dodawanie nowego przepisu nie udało się, spróbuj ponownie" });
            });

        if(saveRecipe) {
            res.json({ message: `Pomyślnie dodano ${name} do przepisów.`});
        }
    })
    .patch('/:id', [passport.authenticate("jwt", { session: false}), validate(validationRecipeSchema)], async (req: Request, res: Response) => {
        const { id } = req.params;
        const { name, description, category, ingredients, todoList, imagePath } = req.body;

        const recipe = await Recipe.findOne({ where: { id } }).catch(err => {
            console.log(err);
            res.status(500)
                .json({ message: "Edycja przepisu nie udała się, spróbuj ponownie" });
        });

        if(!recipe) {
            return res
                .status(400)
                .json( { message: "Przepis o podanym id nie istnieje."})
        }

        recipe.name = name;
        recipe.description = description;
        recipe.category = category;
        recipe.ingredients = ingredients;
        recipe.todoList = todoList;
        recipe.imagePath = imagePath;

        await recipe.save();

        return res
            .status(200)
            .json({ message: "Pomyślnie zaktualizowano przepis."});
    })
    .delete('/:id', passport.authenticate("jwt", { session: false}), async (req, res) => {
        const { id } = req.params;
        const recipe = await Recipe.findOne({ where: { id } }).catch(err => {
            console.log(err);
            res.status(500)
                .json({ message: "Usuwanie przepisu nie udało się, spróbuj ponownie" });
        });

        if(recipe) {
            await recipe.destroy();
            return res
                .status(200)
                .json({ message: "Pomyślnie usunięto przepis."});
        }
        return res
            .status(400)
            .json( { message: "Przepis o podanym id nie istnieje."})
    });
